let port;
let isReading = false;
let writer;
let reader;
let partialMessage = ''; // Stocker les messages partiels
let isSignalRunning = false; // Indiquer si le signal est en cours d'exécution

const outputElement = document.getElementById('output');
const statusElement = document.getElementById('status');
const frequencyInput = document.getElementById('frequency');
const frequencyRange = document.getElementById('frequencyRange');

document.getElementById('connectButton').addEventListener('click', connect);
document.getElementById('disconnectButton').addEventListener('click', disconnect);
document.getElementById('startButton').addEventListener('click', start);
document.getElementById('stopButton').addEventListener('click', stop);
document.getElementById('updateButton').addEventListener('click', updateFrequency);

// Synchroniser le champ de saisie numérique avec le curseur coulissant
frequencyRange.addEventListener('input', () => {
    frequencyInput.value = frequencyRange.value;
});
frequencyInput.addEventListener('input', () => {
    frequencyRange.value = frequencyInput.value;
});

async function connect() {
    if (!("serial" in navigator)) {
        alert("Web Serial API non supportée.");
        return;
    }

    try {
        port = await navigator.serial.requestPort();
        await port.open({ baudRate: 115200 });

        reader = port.readable.getReader();
        writer = port.writable.getWriter();

        readLoop();
        
        updateUIConnected(true);
        statusElement.textContent = "Connecté au port série.";
    } catch (err) {
        statusElement.textContent = "Erreur de connexion au port série: " + err;
    }
}

async function disconnect() {
    if (port) {
        try {
            // Arrêter le signal s'il est en cours d'exécution
            if (isSignalRunning) {
                await stop();
            }

            // Relâcher les verrous avant de fermer le port
            if (reader) {
                await reader.cancel();
                reader.releaseLock();
            }
            if (writer) {
                await writer.close();
                writer.releaseLock();
            }

            await port.close();
            port = null;

            updateUIDisconnected();
            statusElement.textContent = "Déconnecté du port série.";
        } catch (err) {
            console.error('Erreur lors de la déconnexion:', err);
            statusElement.textContent = "Erreur lors de la déconnexion: " + err;
        }
    } else {
        statusElement.textContent = "Aucun port série connecté.";
    }
}

async function start() {
    const frequency = parseInt(frequencyInput.value);
    if (isValidFrequency(frequency)) {
        isSignalRunning = true;
        updateSignalButtons();
        await sendCommand({ command: "START", frequency: frequency });
        statusElement.textContent = "Commande START envoyée.";
    } else {
        alert("Fréquence invalide. Veuillez entrer une fréquence entre 100 et 200000 Hz.");
    }
}

async function stop() {
    isSignalRunning = false;
    updateSignalButtons();
    await sendCommand({ command: "STOP" });
    statusElement.textContent = "Commande STOP envoyée.";
}

async function updateFrequency() {
    const frequency = parseInt(frequencyInput.value);
    if (isValidFrequency(frequency)) {
        await sendCommand({ command: "UPDATE_FREQUENCY", frequency: frequency });
        statusElement.textContent = "Commande UPDATE_FREQUENCY envoyée.";
    } else {
        alert("Fréquence invalide. Veuillez entrer une fréquence entre 100 et 200000 Hz.");
    }
}

function isValidFrequency(frequency) {
    return frequency >= 100 && frequency <= 50000;
}

async function sendCommand(command) {
    if (port) {
        try {
            const message = JSON.stringify(command) + '\n';
            await writer.write(new TextEncoder().encode(message));

            // Attendre l'accusé de réception
            const ack = await waitForAck(command.command, 5000);
            if (ack) {
                statusElement.textContent = `Accusé de réception pour ${command.command} reçu.`;
            } else {
                statusElement.textContent = `Aucun accusé de réception pour ${command.command}.`;
                
                // Réactiver les boutons correspondants si l'accusé de réception n'est pas reçu
                if (command.command === "START") {
                    isSignalRunning = false;
                    updateSignalButtons();
                } else if (command.command === "STOP") {
                    isSignalRunning = true;
                    updateSignalButtons();
                }
            }
        } catch (err) {
            console.error('Erreur d\'envoi de commande:', err);
            statusElement.textContent = "Erreur d'envoi de commande: " + err;
        }
    } else {
        statusElement.textContent = "Non connecté au port série.";
    }
}

async function waitForAck(command, timeout) {
    return new Promise((resolve) => {
        const startTime = Date.now();
        const interval = setInterval(() => {
            if (Date.now() - startTime > timeout) {
                clearInterval(interval);
                resolve(false);
            }

            const ackMessage = outputElement.textContent.split('\n').find(line => {
                try {
                    const msg = JSON.parse(line);
                    return msg.status === "ACK" && msg.command === command;
                } catch (e) {
                    return false;
                }
            });

            if (ackMessage) {
                clearInterval(interval);
                resolve(true);
            }
        }, 100);
    });
}

async function readLoop() {
    isReading = true;
    while (isReading) {
        try {
            const { value, done } = await reader.read();
            if (done) {
                reader.releaseLock();
                break;
            }
            const message = new TextDecoder().decode(value);
            processMessage(message);
        } catch (err) {
            console.error('Erreur de lecture:', err);
            break;
        }
    }
    isReading = false;
}

function processMessage(message) {
    partialMessage += message; // Ajouter le fragment au message partiel

    let endOfMessageIndex;
    while ((endOfMessageIndex = partialMessage.indexOf('\n')) >= 0) {
        const completeMessage = partialMessage.slice(0, endOfMessageIndex);
        partialMessage = partialMessage.slice(endOfMessageIndex + 1);

        try {
            const jsonMessage = JSON.parse(completeMessage);
            outputElement.textContent += JSON.stringify(jsonMessage, null, 2) + '\n';
        } catch (e) {
            console.error('Erreur de lecture JSON:', e);
            outputElement.textContent += "Erreur de lecture JSON: " + completeMessage + '\n';
        }
    }
}

function updateUIConnected(connected) {
    document.getElementById('disconnectButton').disabled = !connected;
    document.getElementById('connectButton').disabled = connected;
    document.getElementById('startButton').disabled = !connected;
    document.getElementById('updateButton').disabled = !connected;
    if (!isSignalRunning) {
        document.getElementById('stopButton').disabled = true;
    }
}

function updateUIDisconnected() {
    document.getElementById('connectButton').disabled = false;
    document.getElementById('disconnectButton').disabled = true;
    document.getElementById('startButton').disabled = true;
    document.getElementById('stopButton').disabled = true;
    document.getElementById('updateButton').disabled = true;
}

function updateSignalButtons() {
    document.getElementById('startButton').disabled = isSignalRunning;
    document.getElementById('stopButton').disabled = !isSignalRunning;
}

navigator.serial.addEventListener('disconnect', (event) => {
    if (port && event.target === port) {
        updateUIDisconnected();
        statusElement.textContent = "Port série déconnecté.";
    }
});
