let port = null; // Reference to the serial port
let reader = null; // Reference to the serial port reader
let currentMessage = ''; // Message currently being received

// Variables to track connection start
let connectionStartTime = null;
let operationDuration = 0; // Operation duration in seconds

// Arrays to store temperature data
let ambientTempData = [];
let objectTempData = [];
let operationDurationData = [];
let labels = [];
// Maximum number of values to display on the chart
const maxDataPoints = 35; // Adjustable

// Global declaration to hold reference to the chart
let myChart = null;

async function connectToSerial() {
    try {
        // Check if the serial port is already open
        if (port && port.readable) {
            console.log("Serial port is already open.");
            return;
        }

        port = await navigator.serial.requestPort();
        console.log("Serial port opened successfully.");

        await port.open({ baudRate: 9600 });
        console.log("Serial port connected.");

        reader = port.readable.getReader();
        console.log("Serial port reader created.");

        // Set the connection start time
        connectionStartTime = new Date();

        while (true) {
            const { value, done } = await reader.read();
            if (done) {
                // The serial port has been closed, disabled, or lost connection.
                reader.releaseLock();
                console.log("Serial port reader released.");
                break;
            }

            // Calculate operation duration since the start of the connection
            const currentTime = new Date();
            operationDuration = (currentTime - connectionStartTime) / 1000; // in seconds

            // Convert data to text string
            const textData = new TextDecoder().decode(value);
            console.log("Data received:", textData);

            // Concatenate new data with previous data
            currentMessage += textData;

            // Check if the message ends with the delimiter ('\n')
            if (currentMessage.endsWith('\n')) {
                // Remove the delimiter from the message
                const cleanedMessage = currentMessage.trim();
                console.log("Complete message:", cleanedMessage);

                // Process the complete message here (assume it's a JSON string).
                try {
                    const data = JSON.parse(cleanedMessage);
                    console.log("Processed data:", data);
                    updateChart(data); // Call function to update chart data
                    document.getElementById('ambientTemp').textContent = data.ambient.toFixed(2);
                    document.getElementById('objectTemp').textContent = data.object.toFixed(2);
                } catch (error) {
                    console.error("JSON parsing error:", error);
                }

                // Reset the current message being received
                currentMessage = '';
            }
        }
    } catch (error) {
        console.error('There was a problem reading data:', error);
    }
}

// Function to stop communication
async function stopCommunication() {
    try {
        // Check if the serial port is open
        if (port && port.readable) {
            // Save data to a text file
            saveDataToFile();

            // Release the serial port read lock
            await reader.cancel();
            await port.close();

            // Reset variables
            port = null;
            reader = null;
            currentMessage = '';
            ambientTempData = [];
            objectTempData = [];
            operationDurationData = [];
            labels = [];

            // Reset the chart
            resetChart();

            // Display confirmation message
            console.log("Communication stopped successfully.");
        } else {
            console.log("No ongoing communication to stop.");
        }
    } catch (error) {
        console.error("Error stopping communication:", error);
    }
}

// Function to create the initial chart
function createChart() {
    const ctx = document.getElementById('temperatureChart').getContext('2d');
    myChart = new Chart(ctx, {
        type: 'line',
        data: {
            labels: [],
            datasets: [{
                label: 'Ambient Temperature (°C)',
                data: [],
                borderColor: 'rgba(255, 99, 132, 1)',
                borderWidth: 1,
                fill: false
            }, {
                label: 'Object Temperature (°C)',
                data: [],
                borderColor: 'rgba(54, 162, 235, 1)',
                borderWidth: 1,
                fill: false
            }]
        },
        options: {

        }
    });
}

// Function to update chart data
function updateChart(data) {
    if (!myChart) {
        createChart(); // Create chart if it doesn't exist yet
    }
    // Remove earliest values if max data points exceeded
    if (myChart.data.labels.length > maxDataPoints) {
        myChart.data.labels.shift();
        myChart.data.datasets[0].data.shift();
        myChart.data.datasets[1].data.shift();
    }
    // Add new data to the chart
    myChart.data.labels.push(operationDuration.toFixed(2));
    myChart.data.datasets[0].data.push(data.ambient.toFixed(2));
    myChart.data.datasets[1].data.push(data.object.toFixed(2));

    // Add new data to arrays

    ambientTempData.push(data.ambient.toFixed(2));
    objectTempData.push(data.object.toFixed(2));
    operationDurationData.push(operationDuration.toFixed(2));

    // Update the chart
    myChart.update();
}

function resetChart() {
    // Reset chart data
    myChart.data.labels = [];
    myChart.data.datasets.forEach((dataset) => {
        dataset.data = [];
    });
    // Update the chart
    myChart.update();
}

function saveDataToFile() {
    // Get current date and time
    const now = new Date();
    const date = now.toLocaleDateString().replaceAll('/', '-'); // Format YYYY-MM-DD
    const time = now.toLocaleTimeString([], { hour12: false }).replaceAll(':', ''); // Format HHMMSS

    // Create filename based on current date and time
    const fileName = `${date} - ${time} - Temperature Data.txt`;

    // Create content to save to text file
    let content = "Temperature Chart Data\n";
    content += "Operation Duration (s),Ambient Temperature (°C),Object Temperature (°C)\n";
    for (let i = 0; i < ambientTempData.length; i++) {
        content += `${operationDurationData[i]},${ambientTempData[i]},${objectTempData[i]}\n`;
    }

    // Create a Blob object with the text content
    const blob = new Blob([content], { type: 'text/plain' });

    // Create a URL object from the Blob
    const url = URL.createObjectURL(blob);

    // Ask the user if they want to download the file
    if (confirm("Do you want to download the file containing temperature data?")) {
        // Create a link to download the text file
        const a = document.createElement('a');
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();

        // Revoke the URL of the Blob object
        URL.revokeObjectURL(url);
    }
}

async function sendCommand() {
    //To be added
}

// Listen for click on "Send" button
document.getElementById('sendButton').addEventListener('click', sendCommand);

// Listen for click on "Stop Communication" button
document.getElementById('stop').addEventListener('click', stopCommunication);

// Listen for click on "Connect to Device" button
document.getElementById('connect').addEventListener('click', connectToSerial);